<?php

/*
 * Project: MinecraftLogFileReader
 * File name: online.php
 * Description: Reads the server logfile of Minecraft to provide useful information
 *
 * @original author Michael Csida, Copyright (C) 2011.
 * @auhtor Benjamin Kristensen
 * @version v 0.4.1 2011/09/02
 * @see The GNU Public License (GPL)
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *
 * Changelog:
 *
 * - v0.1.1 - 2011/06/16 - Upload of first working version (Michael Csida)
 * - v0.1.2 - 2011/06/17 - Minor bugfixes and source code layout changes (Benjamin Kristensen)
 * - v0.2.0 - 2011/06/17 - Added Total Play Time and a new layout (online users are on the top of the list and have an earth-background, offline users have a stone-background) (Michael Csida)
 * - v0.2.1 - 2011/06/18 - Minor bugfixes (Benjamin Kristensen)
 * - v0.3.0 - 2011/06/19 - Added option to display all users (not only users on whitelist), added debug-mode, layout changes (Michael Csida)
 *                         Added refresh rate to auto-reload page (Benjamin Kristensen)
 * - v0.3.1 - 2011/06/19 - Speed improvements (Benjamin Kristensen)
 * - v0.4.0 - 2011/08/18 - Added "Play Time per Day" (Michael Csida)
 * - v0.4.1 - 2011/09/02 - Script now ignores empty lines in the white-list. Fixed bug that Play Time per hour is greater than Total Play Time when the user's first login was within the last 24 hours. (Michael Csida)
 */

/*-----------------------------------------------------------------------
--- VARIABLES YOU CAN / NEED TO CHANGE ----------------------------------
-----------------------------------------------------------------------*/

// Necessary:
$path_to_logs = '/home/minecraft/'; 	// Absolute or relative path to the directory where the logfile (and the whitelist) is located
$logfile = 'server.log'; 				// Name of your server logfile

// Optionally:
$use_whitelist = 0;						// Show users on whitelist ONLY? (will be faster) (1 = yes / 0 = no, show all users)
$whitelist = 'white-list.txt'; 			// Name of your server whitelist file (will be ignored if $use_whitelist is 0)
$display_minute_after = 60; 			// After 60 seconds (default), minutes will be displayed instead of seconds
$display_hour_after = 7200; 			// After 7200 seconds (120 minutes) (default), hours will be displayd instead of minutes
$display_day_after = 172800; 			// After 172800 seconds (48 hours) (default), days will be displayed instead of hours
$remove_offline_after = 604800;			// After 604800 seconds (7 days) (default) offline, the users are removed from the list
$refresh_rate = 60;						// After 60 seconds (default), the page auto-reloads to update the values (set to 0 to disable)
$debug = 0;								// To display debug information at the end of the page (1 = yes / 0 = no)

/*-----------------------------------------------------------------------
--- CODE (DO NOT CHANGE ANYTHING) ---------------------------------------
-----------------------------------------------------------------------*/

// To calculate total runtime of the script
function getmicrotime(){
	list($usec, $sec) = explode(" ",microtime());
	return ((float)$usec + (float)$sec);
}

if ($debug == 1) {
	$time_start = getmicrotime();
}


// Fill variables
$file_logfile = $path_to_logs.$logfile;
$array_logfile = file($file_logfile);

if ($use_whitelist == 1) {
	$file_whitelist = $path_to_logs.$whitelist;
	$array_whitelist = file($file_whitelist);
}

date_default_timezone_set(@date_default_timezone_get());
$current_time = time();


// Temporary LogFile Array with Online/Offline - Info rows only
$h = 0;
foreach($array_logfile as $log_line) {
	if(preg_match('@lost connection: disconnect\.|logged in with entity id@', $log_line)) {
		$array_logfile_temp_onoff[$h] = $log_line;
		$h++;
	}
}


// Create array for username, current userstatus, last change of online/offline status, total play time, play time per day and first login
$h = 0;
if ($use_whitelist == 1) {
	for ($i = 0; $i < count($array_whitelist); $i++) {
		if (rtrim(ltrim($array_whitelist[$i])) != "") {
			$array_users[$h]['name'] = rtrim(ltrim($array_whitelist[$i]));
			$array_users[$h]['status'] = 'UNCLEAR';
			$array_users[$h]['lastchange'] = 0;
			$array_users[$h]['totaltime'] = 0;
			$array_users[$h]['hoursperday'] = 0;
			$array_users[$h]['firstlogin'] = 0;
			$h++;
		}		
	}
} else {
	$array_users_temp = array();
	$h = 0;
	for ($i = 0; $i < count($array_logfile_temp_onoff); $i++) {
		$array_help1 = explode(" [INFO] ", $array_logfile_temp_onoff[$i]);
		$array_help2 = explode(" ", $array_help1[1], 2);
		$flipped_array = array_flip($array_users_temp); 
		if (!isset($flipped_array[$array_help2[0]])) { 
			$array_users_temp[$h] = $array_help2[0];
			$h++; 
		}
	}
	for ($i = 0; $i < count($array_users_temp); $i++) {
		$array_users[$i]['name'] = $array_users_temp[$i];
		$array_users[$i]['status'] = 'UNCLEAR';
		$array_users[$i]['lastchange'] = 0;
		$array_users[$i]['totaltime'] = 0;
		$array_users[$i]['hoursperday'] = 0;
		$array_users[$i]['firstlogin'] = 0;
	}
}
$count_user = count($array_users);
sort($array_users);


// Script: Finding out Online/Offline - Status and set timestamp of last change
$users_checked = 0;
$lines_to_check = count($array_logfile_temp_onoff) - 1;

while (($users_checked < $count_user) && ($lines_to_check >= 0)) {
	$array_temp1 = explode(" [INFO] ", $array_logfile_temp_onoff[$lines_to_check]);
	$array_temp2 = explode(" ", $array_temp1[1], 2);
	for ($i = 0; $i < $count_user; $i++) {
		if ((strtoupper($array_users[$i]['name']) == strtoupper($array_temp2[0])) && $array_users[$i]['status'] == 'UNCLEAR') {
			if (preg_match('#lost connection: disconnect.#', $array_temp2[1])) $array_users[$i]['status'] = 'OFFLINE';
			if (preg_match('#logged in with entity id#', $array_temp2[1])) $array_users[$i]['status'] = 'ONLINE';
			$array_temp3 = explode(" ", $array_temp1[0]);
			$array_temp4 = explode("-", $array_temp3[0]);
			$array_temp5 = explode(":", $array_temp3[1]);
			$array_users[$i]['lastchange'] = mktime($array_temp5[0],$array_temp5[1],$array_temp5[2],$array_temp4[1],$array_temp4[2],$array_temp4[0]);
			$array_users[$i]['name'] = $array_temp2[0];
			$users_checked++;
		}
	}
	$lines_to_check--;
}


// Script: Finding out Total Play Time
$array_logline_timestamp = array();
$array_logline_user = array();
$array_logline_status = array();

for ($i = 0; $i < $count_user; $i++) { // Check one user after another
	$f = 0;
	unset($array_logfile_temp_user);
	for ($g = 0; $g < count($array_logfile_temp_onoff); $g++) { // Create array for online/offline information for this user only
		$array_temp6 = explode(" [INFO] ", $array_logfile_temp_onoff[$g]);
		$array_temp7 = explode(" ", $array_temp6[1], 2);
		if (strtoupper($array_users[$i]['name']) == strtoupper($array_temp7[0])) {
			$array_logfile_temp_user[$f] = $array_logfile_temp_onoff[$g];
			$f++;
		}
	}
	for ($e = 0; $e < count($array_logfile_temp_user); $e++) { // Store all the loglines of the user-array to other arrays once
		$array_logline_temp1 = explode(" [INFO] ", $array_logfile_temp_user[$e]);
		$array_logline_temp2 = explode(" ", $array_logline_temp1[0]);
		$array_logline_temp3 = explode("-", $array_logline_temp2[0]);
		$array_logline_temp4 = explode(":", $array_logline_temp2[1]);
		$array_logline_temp5 = explode(" ", $array_logline_temp1[1], 2);

		$array_logline_timestamp[$e] = mktime($array_logline_temp4[0],$array_logline_temp4[1],$array_logline_temp4[2],$array_logline_temp3[1],$array_logline_temp3[2],$array_logline_temp3[0]);
		if (preg_match('#logged in with entity id#', $array_logline_temp5[1])) {
			$array_logline_status[$e] = 'ONLINE';
		}
		if (preg_match('#lost connection: disconnect.#', $array_logline_temp5[1])) {
			$array_logline_status[$e] = 'OFFLINE';
		}
		$array_logline_user[$e] = $array_logline_temp5[0];
	}
	for ($h = 0; $h < count($array_logfile_temp_user); $h++) { // Calculating Total Played Time
		if ($h == 0) { // First entry of the user logfile
			$temp_current_timestamp = $array_logline_timestamp[$h];
			$array_users[$i]['firstlogin'] = $array_logline_timestamp[$h]; // For "hours per day"
		} else if (($h > 0) && ($h < count($array_logfile_temp_user)-1)){
			if (($array_logline_status[$h] == 'ONLINE') && ($array_logline_status[$h-1] == 'OFFLINE')) {
				$temp_current_timestamp = $array_logline_timestamp[$h];
			}
			if (($array_logline_status[$h] == 'OFFLINE') && ($array_logline_status[$h-1] == 'ONLINE')) {
				$diff = $array_logline_timestamp[$h] - $temp_current_timestamp;
				$array_users[$i]['totaltime'] = $array_users[$i]['totaltime'] + $diff;
			}
		} else if ($h == count($array_logfile_temp_user)-1) { // Last entry of the user logfile
			if ($array_logline_status[$h] == 'ONLINE') {
				$diff = $current_time - $array_logline_timestamp[$h];
				$array_users[$i]['totaltime'] = $array_users[$i]['totaltime'] + $diff;
			}
			if ($array_logline_status[$h] == 'OFFLINE') {
				$diff = $array_logline_timestamp[$h] - $temp_current_timestamp;
				$array_users[$i]['totaltime'] = $array_users[$i]['totaltime'] + $diff;
			}
		}
	}
}

// Calculating hours per day
for ($i = 0; $i < $count_user; $i++) {
	$active_days = ($current_time - $array_users[$i]['firstlogin']) / 60 / 60 / 24;
	$array_users[$i]['hoursperday'] = round(($array_users[$i]['totaltime'] /60 / 60 / $active_days), 3);
}

// Sort user array (online first)
$h=0;
for ($i = 0; $i < $count_user; $i++) {
	if ($array_users[$i]['status'] == 'ONLINE') {
		$array_users_sorted[$h]['name'] = $array_users[$i]['name'];
		$array_users_sorted[$h]['status'] = $array_users[$i]['status'];
		$array_users_sorted[$h]['lastchange'] = $array_users[$i]['lastchange'];
		$array_users_sorted[$h]['totaltime'] = $array_users[$i]['totaltime'];
		$array_users_sorted[$h]['hoursperday'] = $array_users[$i]['hoursperday'];
		$array_users_sorted[$h]['firstlogin'] = $array_users[$i]['firstlogin'];
		$h++;
	}
}
for ($i = 0; $i < $count_user; $i++) {
	if ($array_users[$i]['status'] != 'ONLINE') {
		$array_users_sorted[$h]['name'] = $array_users[$i]['name'];
		$array_users_sorted[$h]['status'] = $array_users[$i]['status'];
		$array_users_sorted[$h]['lastchange'] = $array_users[$i]['lastchange'];
		$array_users_sorted[$h]['totaltime'] = $array_users[$i]['totaltime'];
		$array_users_sorted[$h]['hoursperday'] = $array_users[$i]['hoursperday'];
		$array_users_sorted[$h]['firstlogin'] = $array_users[$i]['firstlogin'];
		$h++;
	}
}

?>


<!-----------------------------------------------------------------------
--- OUTPUT --------------------------------------------------------------
------------------------------------------------------------------------>

<body background="pictures/background.png">

<head>
	<link rel=stylesheet type="text/css" href="style.css">
	<?php if ($refresh_rate > 0) echo "<meta http-equiv=\"refresh\" content=\"".$refresh_rate."\">"; ?>
</head>

<br><br>

<table width=825 align=center cellpadding=3 cellspacing=3 border=0>
	<tr style="background: url(pictures/table_background_01.png); height:35px;">
		<td width=25>&nbsp;</td>
		<td width=125 style="vertical-align: bottom;"><center><span class="vbn12white">Name</span></center></td>
		<td width=225 style="vertical-align: bottom;"><center><span class="vbn12white">Total Play Time</span></center></td>
		<td width=225 style="vertical-align: bottom;"><center><span class="vbn12white">Play Time per Day *</span></center></td>
		<td width=225 style="vertical-align: bottom;"><center><span class="vbn12white">Status</span></center></td>
	</tr>
	<?php
	$count_user_displayed = 0;
	for ($i = 0; $i < $count_user; $i++) {
		if (($current_time - $array_users_sorted[$i]['lastchange'] < $remove_offline_after) || ($array_users_sorted[$i]['status'] == 'ONLINE')) {
			echo "\n<tr style=\"background: url(pictures/";
			if ($array_users_sorted[$i]['status'] == 'ONLINE') echo "table_background_02.png";
			else echo "table_background_03.png";
			echo ")\">\n<td><center><img src=\"pictures/";
			if ($array_users_sorted[$i]['status'] == 'ONLINE') echo "online";
			else if ($array_users_sorted[$i]['status'] == 'OFFLINE') echo "offline";
			else echo "unclear";
			echo ".jpg\"></center></td>\n";
			$temp_totaltime_minutes = sprintf("%01.1f", round($array_users_sorted[$i]['totaltime']/60, 1));
			$temp_totaltime_hours = sprintf("%01.1f", round($array_users_sorted[$i]['totaltime']/60/60, 1));
			$temp_totaltime_days = sprintf("%01.2f", round($array_users_sorted[$i]['totaltime']/60/60/24, 2));
			echo "<td><center><span class=\"vnn10white\">".$array_users_sorted[$i]['name']."</span></center></td>\n";
			echo "<td><center><span class=\"vnn10white\">".$temp_totaltime_hours." hours (".$temp_totaltime_days." days)</center></td>";
			$output_hours = sprintf("%01.2f", round($array_users_sorted[$i]['hoursperday'], 2));
			$output_minutes = sprintf("%01.1f", round(($array_users_sorted[$i]['hoursperday'] * 60), 1));
			if ($output_minutes > $temp_totaltime_minutes) {
				echo "<td><center><span class=\"vnn10white\">".$temp_totaltime_minutes." minutes (".$temp_totaltime_hours." hours)</span></center></td>\n";
			} else {
				echo "<td><center><span class=\"vnn10white\">".$output_minutes." minutes (".$output_hours." hours)</span></center></td>\n";
			}
			echo "<td><span class=\"vnn10white\">";
			if ($array_users_sorted[$i]['status'] == 'ONLINE') echo "Online for ";
			else if ($array_users_sorted[$i]['status'] == 'OFFLINE') echo "Last online: ";
			else echo "Unclear";
			$diff = $current_time - $array_users_sorted[$i]['lastchange'];
			if ($array_users_sorted[$i]['lastchange'] == 0) {
				echo "";
			} else if ($diff >= $display_day_after) {
				$temp = floor($diff/86400);
				echo $temp." day"; if ($temp > 1) echo "s";
			} else if ($diff >= $display_hour_after) {
				$temp = floor($diff/3600);
				echo $temp." hour"; if ($temp > 1) echo "s";
			} else if ($diff >= $display_minute_after) {
				$temp = floor($diff/60);
				echo $temp." minute"; if ($temp > 1) echo "s";
			} else {
				echo $diff." second"; if ($diff > 1) echo "s";
			}
			if ($array_users_sorted[$i]['status'] == 'OFFLINE') echo " ago";		
			echo "</span></td>\n";
			echo "</tr>\n";
			$count_user_displayed++;
		}
	}
	echo "<tr style=\"background: url(pictures/table_background_03.png)\"><td colspan=5><span class=\"vnn10white\">*) Since first Logon</span></td></tr>";
	if ($debug == 1) {
		$time_end = getmicrotime();
		$time = round($time_end - $time_start,4);
		$lines_read = count($array_logfile_temp_onoff) - $lines_to_check - 1;
		$remove_offline_days = sprintf("%01.2f", round($remove_offline_after/60/60/24, 2));
		echo "<tr style=\"background: url(pictures/table_background_03.png)\">";
		echo "<td colspan=5><span class=\"vnn10white\">Page was generated in ".$time." seconds.<br>";
		echo "Users found in "; if ($use_whitelist == 1) echo "whitelist"; else echo "logfile"; echo ": ".$count_user."<br>";
		echo "Users displayed: ".$count_user_displayed." (showing only users who are not offline for more than ".sprintf("%01.2f", round($remove_offline_after/60/60/24, 2))." days)<br>";
		echo "Total logfile lines: ".count($array_logfile)."<br>";
		echo "Logfile lines with online/offline information: ".count($array_logfile_temp_onoff)."<br>";
		echo "Logfile lines read to find out online/offline status: ".$lines_read."</span></td>";
		echo "</tr>";
	}
	?>
</table>

</body>